"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var utils_1 = require("./utils");
var constants_1 = require("./constants");
/**
 * Build Command to run Real Time Database action. All commands call
 * firebase-tools directly, so FIREBASE_TOKEN must exist in environment.
 * @param Cypress - Cypress object
 * @param action - action to run on Firstore (i.e. "add", "delete")
 * @param actionPath - Firestore path where action should be run
 * @param fixturePath - Path to fixture. If object is passed,
 * it is used as options.
 * @param [opts={}] - Options object
 * @param opts.args - Extra arguments to be passed with command
 * @returns Command string to be used with cy.exec
 */
function buildRtdbCommand(Cypress, action, actionPath, fixturePath, opts) {
    if (opts === void 0) { opts = {}; }
    var options = lodash_1.isObject(fixturePath) ? fixturePath : opts;
    var _a = options.args, args = _a === void 0 ? [] : _a;
    var argsWithDefaults = utils_1.addDefaultArgs(Cypress, args);
    var argsStr = utils_1.getArgsString(argsWithDefaults);
    // Add preceding slash if it doesn't already exist (required by firebase-tools)
    var cleanActionPath = actionPath.startsWith('/')
        ? actionPath
        : "/" + actionPath;
    switch (action) {
        case 'remove':
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:" + action + " " + cleanActionPath + argsStr;
        case 'delete':
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:remove " + cleanActionPath + argsStr;
        case 'get': {
            var getDataArgsWithDefaults = utils_1.addDefaultArgs(Cypress, args, {
                disableYes: true,
            });
            if (options.limitToLast) {
                var lastCount = lodash_1.isBoolean(options.limitToLast)
                    ? 1
                    : options.limitToLast;
                if (!options.orderByChild) {
                    getDataArgsWithDefaults.push("--order-by-key --limit-to-last " + lastCount);
                }
                else {
                    getDataArgsWithDefaults.push("--order-by-child " + options.orderByChild + " --limit-to-last " + lastCount);
                }
            }
            var getDataArgsStr = utils_1.getArgsString(getDataArgsWithDefaults);
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:" + action + " " + cleanActionPath + getDataArgsStr;
        }
        default: {
            return constants_1.FIREBASE_TOOLS_BASE_COMMAND + " database:" + action + " " + cleanActionPath + " -d '" + JSON.stringify(options) + "'" + argsStr;
        }
    }
}
exports.default = buildRtdbCommand;
