"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/camelcase */
var lodash_1 = require("lodash");
var path_1 = tslib_1.__importDefault(require("path"));
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var stream_1 = tslib_1.__importDefault(require("stream"));
var child_process_1 = require("child_process");
var fs_1 = require("fs");
var filePaths_1 = require("./filePaths");
var constants_1 = require("./constants");
var logger_1 = require("./logger");
var DEFAULT_BASE_PATH = process.cwd();
/**
 * Get settings from firebaserc file
 * @param filePath - Path for file
 * @returns Firebase settings object
 */
function readJsonFile(filePath) {
    if (!fs_1.existsSync(filePath)) {
        return {};
    }
    try {
        return JSON.parse(fs_1.readFileSync(filePath, 'utf8'));
    }
    catch (err) {
        logger_1.error("Unable to parse " + chalk_1.default.cyan(filePath.replace(DEFAULT_BASE_PATH, '')) + " - JSON is most likely not valid");
        return {};
    }
}
exports.readJsonFile = readJsonFile;
/**
 * Get environment slug
 * @returns Environment slug
 */
function getEnvironmentSlug() {
    return (process.env.CI_ENVIRONMENT_SLUG || process.env.CI_COMMIT_REF_SLUG || 'stage');
}
/**
 * Get prefix for current environment based on environment vars available
 * within CI. Falls back to staging (i.e. STAGE)
 * @param envName - Environment option
 * @returns Environment prefix string
 */
function getEnvPrefix(envName) {
    var envSlug = envName || getEnvironmentSlug();
    return envSlug.toUpperCase() + "_";
}
exports.getEnvPrefix = getEnvPrefix;
/**
 * Get path to local service account
 * @param envName - Environment option
 * @returns Path to service account
 */
function getServiceAccountPath(envName) {
    var withSuffix = path_1.default.join(DEFAULT_BASE_PATH, "serviceAccount-" + (envName || '') + ".json");
    if (fs_1.existsSync(withSuffix)) {
        return withSuffix;
    }
    return path_1.default.join(DEFAULT_BASE_PATH, 'serviceAccount.json');
}
/**
 * Get cypress folder path from cypress.json config file or fallback to
 * default folder path ('cypress')
 * @returns Path of folder containing cypress folders like "integration"
 */
function getCypressFolderPath() {
    var cypressConfig = readJsonFile(filePaths_1.TEST_CONFIG_FILE_PATH); // eslint-disable-line no-use-before-define
    var integrationTestsFolderPath = lodash_1.get(cypressConfig, 'integrationFolder');
    return integrationTestsFolderPath
        ? integrationTestsFolderPath
            .split('/')
            .slice(0, -1) // Drop last item (equivalent to dropRight)
            .join('/')
        : constants_1.DEFAULT_TEST_FOLDER_PATH;
}
exports.getCypressFolderPath = getCypressFolderPath;
/**
 * Get path to cypress config file
 * @returns Path to cypress config file
 */
function getCypressConfigPath() {
    var cypressFolderPath = getCypressFolderPath();
    var cypressConfigFilePath = path_1.default.join(cypressFolderPath, constants_1.DEFAULT_CONFIG_FILE_NAME);
    return cypressConfigFilePath;
}
exports.getCypressConfigPath = getCypressConfigPath;
/**
 * Get environment variable based on the current CI environment
 * @param varNameRoot - variable name without the environment prefix
 * @param envName - Environment option
 * @returns Value of the environment variable
 * @example
 * envVarBasedOnCIEnv('FIREBASE_PROJECT_ID')
 * // => 'fireadmin-stage' (value of 'STAGE_FIREBASE_PROJECT_ID' environment var)
 */
function envVarBasedOnCIEnv(varNameRoot, envName) {
    var prefix = getEnvPrefix(envName);
    var combined = "" + prefix + varNameRoot;
    var localConfigFilePath = getCypressConfigPath();
    // Config file used for environment (local, containers) from main test path ({integrationFolder}/config.json)
    if (fs_1.existsSync(localConfigFilePath)) {
        var configObj = readJsonFile(localConfigFilePath);
        return configObj[combined] || configObj[varNameRoot];
    }
    // Config file used for environment from main cypress environment file (cypress.env.json)
    if (fs_1.existsSync(filePaths_1.TEST_ENV_FILE_PATH)) {
        var configObj = readJsonFile(filePaths_1.TEST_ENV_FILE_PATH);
        return configObj[combined] || configObj[varNameRoot];
    }
    // CI Environment (environment variables loaded directly)
    return process.env[combined] || process.env[varNameRoot];
}
exports.envVarBasedOnCIEnv = envVarBasedOnCIEnv;
/**
 * Get parsed value of environment variable. Useful for environment variables
 * which have characters that need to be escaped.
 * @param varNameRoot - variable name without the environment prefix
 * @param envName - Environment option
 * @returns Value of the environment variable
 * @example
 * getParsedEnvVar('FIREBASE_PRIVATE_KEY_ID')
 * // => 'fireadmin-stage' (parsed value of 'STAGE_FIREBASE_PRIVATE_KEY_ID' environment var)
 */
function getParsedEnvVar(varNameRoot, envName) {
    var val = envVarBasedOnCIEnv(varNameRoot, envName);
    var prefix = getEnvPrefix();
    var combinedVar = "" + prefix + varNameRoot;
    if (!val) {
        logger_1.error(chalk_1.default.cyan(combinedVar) + " not found, make sure it is set within environment variables.");
    }
    try {
        if (typeof val === 'string') {
            return JSON.parse(val);
        }
        return val;
    }
    catch (err) {
        logger_1.error("Error parsing " + combinedVar);
        return val;
    }
}
/**
 * Get service account from either local file or environment variables
 * @param envSlug - Environment option
 * @returns Service account object
 */
function getServiceAccount(envSlug) {
    var serviceAccountPath = getServiceAccountPath(envSlug);
    // Check for local service account file (Local dev)
    if (fs_1.existsSync(serviceAccountPath)) {
        return readJsonFile(serviceAccountPath); // eslint-disable-line global-require, import/no-dynamic-require
    }
    logger_1.info("Service account does not exist at path: \"" + chalk_1.default.cyan(serviceAccountPath.replace(DEFAULT_BASE_PATH + "/", '')) + "\" falling back to environment variables...");
    // Use environment variables (CI)
    var serviceAccountEnvVar = envVarBasedOnCIEnv('SERVICE_ACCOUNT', envSlug);
    if (serviceAccountEnvVar) {
        if (typeof serviceAccountEnvVar === 'string') {
            try {
                return JSON.parse(serviceAccountEnvVar);
            }
            catch (err) {
                logger_1.warn('Issue parsing SERVICE_ACCOUNT environment variable from string to object, returning string');
            }
        }
        return serviceAccountEnvVar;
    }
    var clientId = envVarBasedOnCIEnv('FIREBASE_CLIENT_ID', envSlug);
    if (clientId) {
        logger_1.warn('"FIREBASE_CLIENT_ID" will override FIREBASE_TOKEN for auth when calling firebase-tools - this may cause unexepected behavior');
    }
    return {
        type: 'service_account',
        project_id: envVarBasedOnCIEnv('FIREBASE_PROJECT_ID', envSlug),
        private_key_id: envVarBasedOnCIEnv('FIREBASE_PRIVATE_KEY_ID', envSlug),
        private_key: getParsedEnvVar('FIREBASE_PRIVATE_KEY', envSlug),
        client_email: envVarBasedOnCIEnv('FIREBASE_CLIENT_EMAIL', envSlug),
        client_id: clientId,
        auth_uri: 'https://accounts.google.com/o/oauth2/auth',
        token_uri: 'https://accounts.google.com/o/oauth2/token',
        auth_provider_x509_cert_url: 'https://www.googleapis.com/oauth2/v1/certs',
        client_x509_cert_url: envVarBasedOnCIEnv('FIREBASE_CERT_URL', envSlug),
    };
}
exports.getServiceAccount = getServiceAccount;
/**
 * Run a bash command using spawn pipeing the results to the main process
 * @param runOptions - Options for command run
 * @param runOptions.command - Command to be executed
 * @param runOptions.args - Command arguments
 * @returns Resolves with results of running the command
 * @private
 */
function runCommand(runOptions) {
    var beforeMsg = runOptions.beforeMsg, successMsg = runOptions.successMsg, command = runOptions.command, errorMsg = runOptions.errorMsg, args = runOptions.args, _a = runOptions.pipeOutput, pipeOutput = _a === void 0 ? true : _a;
    if (beforeMsg)
        logger_1.info(beforeMsg);
    return new Promise(function (resolve, reject) {
        var child = child_process_1.spawn(command, args);
        var output;
        var error;
        var customStream = new stream_1.default.Writable();
        var customErrorStream = new stream_1.default.Writable();
        /* eslint-disable no-underscore-dangle */
        customStream._write = function (data) {
            var _a;
            var argv = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                argv[_i - 1] = arguments[_i];
            }
            output += data;
            if (pipeOutput) {
                (_a = process.stdout)._write.apply(_a, tslib_1.__spread([data], argv));
            }
        };
        customErrorStream._write = function (data) {
            var _a;
            var argv = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                argv[_i - 1] = arguments[_i];
            }
            error += data;
            if (pipeOutput) {
                (_a = process.stderr)._write.apply(_a, tslib_1.__spread([data], argv));
            }
        };
        /* eslint-enable no-underscore-dangle */
        // Pipe errors and console output to main process
        child.stdout.pipe(customStream);
        child.stderr.pipe(customErrorStream);
        // When child exits resolve or reject based on code
        child.on('exit', function (code) {
            if (code !== 0) {
                // Resolve for npm warnings
                if (output && output.indexOf('npm WARN') !== -1) {
                    return resolve(successMsg || output);
                }
                if (errorMsg) {
                    console.log(errorMsg); // eslint-disable-line no-console
                }
                reject(error || output);
            }
            else {
                // resolve(null, stdout)
                if (successMsg)
                    logger_1.info(successMsg);
                // Remove leading undefined from response
                if (output && output.indexOf('undefined') === 0) {
                    resolve(successMsg || output.replace('undefined', ''));
                }
                else {
                    resolve(successMsg || output);
                }
            }
        });
    });
}
exports.runCommand = runCommand;
process.env.FORCE_COLOR = 'true';
