"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/camelcase */
var constants_1 = require("./constants");
/**
 * Async await wrapper for easy error handling
 * @param promise - Promise to wrap responses of in array
 * @param errorExt - Extension for error
 * @returns Resolves and rejects with an array
 */
function to(promise, errorExt) {
    return promise
        .then(function (data) { return [null, data]; })
        .catch(function (err) {
        if (errorExt) {
            Object.assign(err, errorExt);
        }
        return [err, undefined];
    });
}
exports.to = to;
/**
 * Convert slash path to Firestore reference
 * @param firestoreInstance - Instance on which to
 * create ref
 * @param slashPath - Path to convert into firestore refernce
 * @param options - Options object
 * @returns Ref at slash path
 */
function slashPathToFirestoreRef(firestoreInstance, slashPath, options) {
    var ref = firestoreInstance;
    var srcPathArr = slashPath.split('/');
    srcPathArr.forEach(function (pathSegment) {
        if (ref.collection) {
            ref = ref.collection(pathSegment);
        }
        else if (ref.doc) {
            ref = ref.doc(pathSegment);
        }
        else {
            throw new Error("Invalid slash path: " + slashPath);
        }
    });
    // Apply limit to query if it exists
    if (options && options.limit && typeof ref.limit === 'function') {
        ref = ref.limit(options.limit);
    }
    return ref;
}
exports.slashPathToFirestoreRef = slashPathToFirestoreRef;
/**
 * Create command arguments string from an array of arguments by joining them
 * with a space including a leading space. If no args provided, empty string
 * is returned
 * @param args - Command arguments to convert into a string
 * @returns Arguments section of command string
 */
function getArgsString(args) {
    return args && args.length ? " " + args.join(' ') : '';
}
exports.getArgsString = getArgsString;
/**
 * Add default Firebase arguments to arguments array.
 * @param Cypress - Cypress object
 * @param args - arguments array
 * @param [opts={}] - Options object
 * @param opts.token - Firebase CI token to pass as the token argument
 * @param [opts.disableYes=false] - Whether or not to disable the yes argument
 * @returns Default args list
 */
function addDefaultArgs(Cypress, args, opts) {
    var _a = opts || {}, _b = _a.disableYes, disableYes = _b === void 0 ? false : _b, token = _a.token;
    var newArgs = tslib_1.__spread(args);
    // TODO: Load this in a way that understands environment. Currently this will
    // go to the first project id that is defined, not which one should be used
    // for the specified environment
    var projectId = Cypress.env('firebaseProjectId') ||
        Cypress.env('FIREBASE_PROJECT_ID') ||
        Cypress.env('STAGE_FIREBASE_PROJECT_ID');
    // Include project id command so command runs on the current project
    if (!newArgs.includes('-P') || !newArgs.includes(projectId)) {
        newArgs.push('-P');
        newArgs.push(projectId);
    }
    var tokenFromEnv = Cypress.env('FIREBASE_TOKEN');
    // Include token if it exists in environment
    if (!newArgs.includes('--token') && (token || tokenFromEnv)) {
        newArgs.push('--token');
        newArgs.push(token || tokenFromEnv);
    }
    // Add Firebase's automatic approval argument if it is not already in newArgs
    if (!disableYes && !newArgs.includes(constants_1.FIREBASE_TOOLS_YES_ARGUMENT)) {
        newArgs.push(constants_1.FIREBASE_TOOLS_YES_ARGUMENT);
    }
    return newArgs;
}
exports.addDefaultArgs = addDefaultArgs;
process.env.FORCE_COLOR = 'true';
/**
 * Check to see if the provided value is a promise object
 * @param valToCheck - Value to be checked for Promise qualities
 * @returns Whether or not provided value is a promise
 */
function isPromise(valToCheck) {
    return valToCheck && typeof valToCheck.then === 'function';
}
exports.isPromise = isPromise;
/**
 * Escape shell command arguments and join them to a single string
 * @param a - List of arguments to escape
 * @returns Command string with arguments escaped
 */
function shellescape(a) {
    var ret = [];
    a.forEach(function (s) {
        if (/[^A-Za-z0-9_/:=-]/.test(s)) {
            // eslint-disable-line no-useless-escape
            s = "'" + s.replace(/'/g, "'\\''") + "'"; // eslint-disable-line no-param-reassign
            s = s // eslint-disable-line no-param-reassign
                .replace(/^(?:'')+/g, '') // unduplicate single-quote at the beginning
                .replace(/\\'''/g, "\\'"); // remove non-escaped single-quote if there are enclosed between 2 escaped
        }
        ret.push(s);
    });
    return ret.join(' ');
}
exports.shellescape = shellescape;
